# THIS IS OLD DO NOT USE

from .cpAST import get_ast, print_ast
from .cnodevisitor import NodeVisitor

def readast(node, depth, calc):
    calc(node, depth)
    for child in node.get_children():
        print_ast(child, depth + 1, calc)
        
def translate(file_path):
    ast = get_ast(file_path)
    print_ast(ast)

    newNodeVisitor = NodeVisitor()
    newNodeVisitor.visit_node(ast)
    
    print(newNodeVisitor.get_lua_code())


"""Boolean operation description"""
import ast


_DEFAULT_FORMAT = "{left} {operation} {right}"


class BooleanOperationDesc:
    """Binary operation description"""

    OPERATION = {
        ast.And: {
            "value": "and",
            "format": _DEFAULT_FORMAT,
        },
        ast.Or: {
            "value": "or",
            "format": _DEFAULT_FORMAT,
        },
    }



"""Class to store the python code context"""
from .symbolsstack import SymbolsStack
from .tokenendmode import TokenEndMode


class Context:
    """Class to store the python code context"""
    def __init__(self, values=None):
        values = values if values is not None else {
            "token_end_mode": TokenEndMode.LINE_FEED,
            "class_name": "",
            "locals": SymbolsStack(),
            "globals": SymbolsStack(),  # Not working yet
            "loop_label_name": "",
            "docstring": False,
        }

        self.ctx_stack = [values]

    def last(self):
        """Return actual context state"""
        return self.ctx_stack[-1]

    def push(self, values):
        """Push new context state with new values"""
        value = self.ctx_stack[-1].copy()
        value.update(values)
        self.ctx_stack.append(value)

    def pop(self):
        """Pop last context state"""
        assert len(self.ctx_stack) > 1, "Pop context failed. This is a last context in the stack."
        return self.ctx_stack.pop()



# THIS IS OLD DO NOT USE

"""
Dedicated to parsing C/C++ code into ASTs using libclang.
Pass the code on to the C(++) translator.
"""

import clang.cindex as clang

def get_ast(file_path):
    index = clang.Index.create()
    translation_unit = index.parse(file_path)
    return translation_unit.cursor

def print_ast(node, depth=0):
    print('  ' * depth + str(node.kind) + ' : ' + node.spelling)
    for child in node.get_children():
        print_ast(child, depth + 1)
